@@ -4,7 +4,7 @@
 * Author:      Vadim Zeitlin
 * Modified by:
 * Created:     09.08.00
-* RCS-ID:      $Id: chkconf.h,v 1.75 2004/09/29 08:03:10 ABX Exp $
+* RCS-ID:      $Id: chkconf.h,v 1.101 2005/03/10 19:06:26 VZ Exp $
 * Copyright:   (c) 2000 Vadim Zeitlin <vadim@wxwidgets.org>
 * Licence:     wxWindows licence
 */
@@ -15,7 +15,11 @@
    Compiler-specific checking.
  */
 
-#if defined(__WXMSW__)
+#if defined(__WXPALMOS__)
+#  include "wx/palmos/chkconf.h"
+#elif defined(__WXWINCE__)
+#  include "wx/msw/wince/chkconf.h"
+#elif defined(__WXMSW__)
 #  include "wx/msw/chkconf.h"
 #elif defined(__WXMAC__)
 #  include "wx/mac/chkconf.h"
@@ -38,6 +42,27 @@
 #   define wxUSE_GUI 1
 #endif /* !defined(wxUSE_GUI) */
 
+/*
+    If we're compiling without support for threads/exceptions we have to
+    disable the corresponding features.
+ */
+#ifdef wxNO_THREADS
+#   undef wxUSE_THREADS
+#   define wxUSE_THREADS 0
+#endif /* wxNO_THREADS */
+
+#ifdef wxNO_EXCEPTIONS
+#   undef wxUSE_EXCEPTIONS
+#   define wxUSE_EXCEPTIONS 0
+#endif /* wxNO_EXCEPTIONS */
+
+/* we also must disable exceptions if compiler doesn't support them */
+#if defined(_MSC_VER) && !defined(_CPPUNWIND)
+#   undef wxUSE_EXCEPTIONS
+#   define wxUSE_EXCEPTIONS 0
+#endif /* VC++ without exceptions support */
+
+
 /*
    tests for non GUI features
  */
@@ -153,6 +178,14 @@
 #   endif
 #endif /* !defined(wxUSE_REGEX) */
 
+#ifndef wxUSE_STDPATHS
+#   ifdef wxABORT_ON_CONFIG_ERROR
+#       error "wxUSE_STDPATHS must be defined."
+#   else
+#       define wxUSE_STDPATHS 1
+#   endif
+#endif /* !defined(wxUSE_STDPATHS) */
+
 #ifndef wxUSE_XML
 #   ifdef wxABORT_ON_CONFIG_ERROR
 #       error "wxUSE_XML must be defined."
@@ -339,6 +372,14 @@
 #   endif
 #endif /* !defined(wxUSE_DATAOBJ) */
 
+#ifndef wxUSE_DATEPICKCTRL
+#   ifdef wxABORT_ON_CONFIG_ERROR
+#       error "wxUSE_DATEPICKCTRL must be defined."
+#   else
+#       define wxUSE_DATEPICKCTRL 0
+#   endif
+#endif /* !defined(wxUSE_DATEPICKCTRL) */
+
 #ifndef wxUSE_DISPLAY
 #   ifdef wxABORT_ON_CONFIG_ERROR
 #       error "wxUSE_DISPLAY must be defined."
@@ -922,6 +963,25 @@
 #   endif
 #endif  /* wxUSE_DYNLIB_CLASS */
 
+#if wxUSE_ZIPSTREAM
+#   if !wxUSE_ZLIB
+#       ifdef wxABORT_ON_CONFIG_ERROR
+#           error "wxZip requires wxZlib"
+#       else
+#           undef wxUSE_ZLIB
+#           define wxUSE_ZLIB 1
+#       endif
+#   endif
+#   if !wxUSE_ARCHIVE_STREAMS
+#       ifdef wxABORT_ON_CONFIG_ERROR
+#           error "wxZip requires wxArchive"
+#       else
+#           undef wxUSE_ARCHIVE_STREAMS
+#           define wxUSE_ARCHIVE_STREAMS 1
+#       endif
+#   endif
+#endif /* wxUSE_ZIPSTREAM */
+
 /* the rest of the tests is for the GUI settings only */
 #if wxUSE_GUI
 
@@ -1009,6 +1069,18 @@
 #   endif
 #endif /* wxUSE_CHOICEBOOK */
 
+/* don't attempt to use native status bar on the platforms not having it */
+#ifndef wxUSE_NATIVE_STATUSBAR
+#   define wxUSE_NATIVE_STATUSBAR 0
+#elif wxUSE_NATIVE_STATUSBAR
+#   if defined(__WXUNIVERSAL__) || !( defined(__WXMSW__) || \
+                                      defined(__WXMAC__) || \
+                                      defined(__WXPALMOS__) )
+#       undef wxUSE_NATIVE_STATUSBAR
+#       define wxUSE_NATIVE_STATUSBAR 0
+#   endif
+#endif
+
 /* wxUniv-specific dependencies */
 #if defined(__WXUNIVERSAL__)
 #   if (wxUSE_COMBOBOX || wxUSE_MENUS) && !wxUSE_POPUPWIN
@@ -1142,6 +1214,7 @@
 #           endif
 #       endif
 #   endif  /* wxUSE_DYNAMIC_LOADER */
+
 #endif /* wxMSW */
 
 /* wxMAC-specific dependencies */
@@ -1228,6 +1301,17 @@
 #   endif
 #endif /* wxUSE_CALENDARCTRL */
 
+#if wxUSE_DATEPICKCTRL
+#   if !wxUSE_DATETIME
+#       ifdef wxABORT_ON_CONFIG_ERROR
+#           error "wxDatePickerCtrl requires wxUSE_DATETIME"
+#       else
+#           undef wxUSE_DATETIME
+#           define wxUSE_DATETIME 1
+#       endif
+#   endif
+#endif /* wxUSE_DATEPICKCTRL */
+
 #if wxUSE_CHECKLISTBOX
 #   if !wxUSE_LISTBOX
 #        ifdef wxABORT_ON_CONFIG_ERROR
@@ -1271,7 +1355,7 @@
 #endif /* wxUSE_HELP */
 
 #if wxUSE_WXHTML_HELP
-#   if !wxUSE_HELP || !wxUSE_HTML || !wxUSE_COMBOBOX || !wxUSE_NOTEBOOK
+#   if !wxUSE_HELP || !wxUSE_HTML || !wxUSE_COMBOBOX || !wxUSE_NOTEBOOK || !wxUSE_SPINCTRL
 #       ifdef wxABORT_ON_CONFIG_ERROR
 #           error "Built in help controller can't be compiled"
 #       else
@@ -1283,6 +1367,8 @@
 #           define wxUSE_COMBOBOX 1
 #           undef wxUSE_NOTEBOOK
 #           define wxUSE_NOTEBOOK 1
+#           undef wxUSE_SPINCTRL
+#           define wxUSE_SPINCTRL 1
 #       endif
 #   endif
 #endif /* wxUSE_WXHTML_HELP */
@@ -1456,9 +1542,21 @@
 #   endif
 #endif /* wxUSE_FILEDLG */
 
+#if !wxUSE_GAUGE || !wxUSE_BUTTON
+#   if wxUSE_PROGRESSDLG && !defined(__WXPALMOS__)
+#       ifdef wxABORT_ON_CONFIG_ERROR
+#           error "Generic progress dialog requires wxUSE_GAUGE and wxUSE_BUTTON"
+#       else
+#           undef wxUSE_GAUGE
+#           undef wxUSE_BUTTON
+#           define wxUSE_GAUGE 1
+#           define wxUSE_BUTTON 1
+#       endif
+#   endif
+#endif /* !wxUSE_GAUGE */
+
 #if !wxUSE_BUTTON
-#   if wxUSE_PROGRESSDLG || \
-       wxUSE_FONTDLG || \
+#   if wxUSE_FONTDLG || \
        wxUSE_FILEDLG || \
        wxUSE_CHOICEDLG || \
        wxUSE_NUMBERDLG || \
@@ -1473,7 +1571,7 @@
 #           define wxUSE_BUTTON 1
 #       endif
 #   endif
-#endif /* wxUSE_PROGRESSDLG */
+#endif /* !wxUSE_BUTTON */
 
 #if !wxUSE_TOOLBAR
 #   if wxUSE_TOOLBAR_NATIVE
@@ -1517,13 +1615,19 @@
 #endif
 
 #if wxUSE_RADIOBOX
-#   if !wxUSE_RADIOBTN || !wxUSE_STATBOX
+#   if !wxUSE_RADIOBTN
 #        ifdef wxABORT_ON_CONFIG_ERROR
-#            error "wxUSE_RADIOBOX requires wxUSE_RADIOBTN and wxUSE_STATBOX"
+#            error "wxUSE_RADIOBOX requires wxUSE_RADIOBTN"
 #        else
 #            undef wxUSE_RADIOBTN
-#            undef wxUSE_STATBOX
 #            define wxUSE_RADIOBTN 1
+#        endif
+#   endif
+#   if !wxUSE_STATBOX && !defined(__WXPALMOS__)
+#        ifdef wxABORT_ON_CONFIG_ERROR
+#            error "wxUSE_RADIOBOX requires wxUSE_STATBOX"
+#        else
+#            undef wxUSE_STATBOX
 #            define wxUSE_STATBOX 1
 #        endif
 #   endif
@@ -1553,17 +1657,6 @@
 #    endif
 #endif /* wxUSE_LOG_DIALOG */
 
-/* I wonder if we shouldn't just remove all occurrences of
-   wxUSE_DYNAMIC_CLASSES from the sources? */
-#if !defined(wxUSE_DYNAMIC_CLASSES) || !wxUSE_DYNAMIC_CLASSES
-#   ifdef wxABORT_ON_CONFIG_ERROR
-#       error "wxUSE_DYNAMIC_CLASSES must be defined as 1"
-#   else
-#       undef wxUSE_DYNAMIC_CLASSES
-#       define wxUSE_DYNAMIC_CLASSES 1
-#   endif
-#endif /* wxUSE_DYNAMIC_CLASSES */
-
 #if wxUSE_CLIPBOARD && !wxUSE_DATAOBJ
 #   ifdef wxABORT_ON_CONFIG_ERROR
 #       error "wxClipboard requires wxDataObject"
@@ -1591,5 +1684,14 @@
 #   endif
 #endif /* wxUSE_XRC */
 
+#if wxUSE_SOCKETS && !wxUSE_STOPWATCH
+#   ifdef wxABORT_ON_CONFIG_ERROR
+#       error "wxUSE_SOCKETS requires wxUSE_STOPWATCH"
+#   else
+#       undef wxUSE_SOCKETS
+#       define wxUSE_SOCKETS 0
+#   endif
+#endif /* wxUSE_SOCKETS */
+
 #endif /* wxUSE_GUI */
 
