@@ -5,7 +5,7 @@
 // Author:      Jeffrey C. Ollie <jeff@ollie.clive.ia.us>, Vadim Zeitlin
 // Modified by:
 // Created:     10.02.99
-// RCS-ID:      $Id: longlong.h,v 1.55 2004/05/23 20:50:23 JS Exp $
+// RCS-ID:      $Id: longlong.h,v 1.60 2005/02/16 09:35:10 ABX Exp $
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -115,6 +115,9 @@ public:
         m_ll = ((wxLongLong_t) hi) << 32;
         m_ll |= (wxLongLong_t) lo;
     }
+#if wxUSE_LONGLONG_WX
+    wxLongLongNative(wxLongLongWx ll);
+#endif
 
     // default copy ctor is ok
 
@@ -124,6 +127,10 @@ public:
         // from native 64 bit integer
     wxLongLongNative& operator=(wxLongLong_t ll)
         { m_ll = ll; return *this; }
+#if wxUSE_LONGLONG_WX
+    wxLongLongNative& operator=(wxLongLongWx ll);
+#endif
+
 
         // from double: this one has an explicit name because otherwise we
         // would have ambiguity with "ll = int" and also because we don't want
@@ -294,9 +301,13 @@ public:
 
 #if wxUSE_STD_IOSTREAM
         // input/output
-    friend wxSTD ostream& operator<<(wxSTD ostream&, const wxLongLongNative&);
+    friend WXDLLIMPEXP_BASE
+    wxSTD ostream& operator<<(wxSTD ostream&, const wxLongLongNative&);
 #endif
 
+    friend WXDLLIMPEXP_BASE
+    wxString& operator<<(wxString&, const wxLongLongNative&);
+
 private:
     wxLongLong_t  m_ll;
 };
@@ -309,13 +320,13 @@ public:
         // default ctor initializes to 0
     wxULongLongNative() : m_ll(0) { }
         // from long long
-    wxULongLongNative(unsigned wxLongLong_t ll) : m_ll(ll) { }
+    wxULongLongNative(wxULongLong_t ll) : m_ll(ll) { }
         // from 2 longs
     wxULongLongNative(unsigned long hi, unsigned long lo) : m_ll(0)
     {
         // assign first to avoid precision loss!
-        m_ll = ((unsigned wxLongLong_t) hi) << 32;
-        m_ll |= (unsigned wxLongLong_t) lo;
+        m_ll = ((wxULongLong_t) hi) << 32;
+        m_ll |= (wxULongLong_t) lo;
     }
 
     // default copy ctor is ok
@@ -324,7 +335,7 @@ public:
 
     // assignment operators
         // from native 64 bit integer
-    wxULongLongNative& operator=(unsigned wxLongLong_t ll)
+    wxULongLongNative& operator=(wxULongLong_t ll)
         { m_ll = ll; return *this; }
 
     // assignment operators from wxULongLongNative is ok
@@ -338,7 +349,7 @@ public:
         { return (unsigned long)m_ll; }
 
         // convert to native ulong long
-    unsigned wxLongLong_t GetValue() const { return m_ll; }
+    wxULongLong_t GetValue() const { return m_ll; }
 
         // convert to ulong with range checking in the debug mode (only!)
     unsigned long ToULong() const
@@ -356,9 +367,9 @@ public:
     wxULongLongNative& operator+=(const wxULongLongNative& ll)
         { m_ll += ll.m_ll; return *this; }
 
-    wxULongLongNative operator+(const unsigned wxLongLong_t ll) const
+    wxULongLongNative operator+(const wxULongLong_t ll) const
         { return wxULongLongNative(m_ll + ll); }
-    wxULongLongNative& operator+=(const unsigned wxLongLong_t ll)
+    wxULongLongNative& operator+=(const wxULongLong_t ll)
         { m_ll += ll; return *this; }
 
         // pre increment
@@ -375,9 +386,9 @@ public:
     wxULongLongNative& operator-=(const wxULongLongNative& ll)
         { m_ll -= ll.m_ll; return *this; }
 
-    wxULongLongNative operator-(const unsigned wxLongLong_t ll) const
+    wxULongLongNative operator-(const wxULongLong_t ll) const
         { return wxULongLongNative(m_ll - ll); }
-    wxULongLongNative& operator-=(const unsigned wxLongLong_t ll)
+    wxULongLongNative& operator-=(const wxULongLong_t ll)
         { m_ll -= ll; return *this; }
 
         // pre decrement
@@ -477,11 +488,15 @@ public:
 
 #if wxUSE_STD_IOSTREAM
         // input/output
-    friend wxSTD ostream& operator<<(wxSTD ostream&, const wxULongLongNative&);
+    friend WXDLLIMPEXP_BASE
+    wxSTD ostream& operator<<(wxSTD ostream&, const wxULongLongNative&);
 #endif
 
+    friend WXDLLIMPEXP_BASE
+    wxString& operator<<(wxString&, const wxULongLongNative&);
+
 private:
-    unsigned wxLongLong_t  m_ll;
+    wxULongLong_t  m_ll;
 };
 
 #endif // wxUSE_LONGLONG_NATIVE
@@ -623,6 +638,10 @@ public:
     // comparison
     bool operator==(const wxLongLongWx& ll) const
         { return m_lo == ll.m_lo && m_hi == ll.m_hi; }
+#if wxUSE_LONGLONG_NATIVE
+    bool operator==(const wxLongLongNative& ll) const
+        { return m_lo == ll.GetLo() && m_hi == ll.GetHi(); }
+#endif
     bool operator!=(const wxLongLongWx& ll) const
         { return !(*this == ll); }
     bool operator<(const wxLongLongWx& ll) const;
@@ -665,9 +684,13 @@ public:
     void *asArray() const;
 
 #if wxUSE_STD_IOSTREAM
-    friend wxSTD ostream& operator<<(wxSTD ostream&, const wxLongLongWx&);
+    friend WXDLLIMPEXP_BASE
+    wxSTD ostream& operator<<(wxSTD ostream&, const wxLongLongWx&);
 #endif // wxUSE_STD_IOSTREAM
 
+    friend WXDLLIMPEXP_BASE
+    wxString& operator<<(wxString&, const wxLongLongWx&);
+
 private:
     // long is at least 32 bits, so represent our 64bit number as 2 longs
 
@@ -717,6 +740,14 @@ public:
 #endif // wxLONGLONG_TEST_MODE
     }
 
+    // from signed to unsigned
+    wxULongLongWx(wxLongLongWx ll)
+    {
+        wxASSERT(ll.GetHi() >= 0);
+        m_hi = (unsigned long)ll.GetHi();
+        m_lo = ll.GetLo();
+    }
+
     // default copy ctor is ok in both cases
 
     // no dtor
@@ -767,8 +798,8 @@ public:
         // post increment operator
     wxULongLongWx& operator++(int) { return ++(*this); }
 
-        // subraction (FIXME: should return wxLongLong)
-    wxULongLongWx operator-(const wxULongLongWx& ll) const;
+        // substraction
+    wxLongLongWx operator-(const wxULongLongWx& ll) const;
     wxULongLongWx& operator-=(const wxULongLongWx& ll);
 
         // pre decrement operator
@@ -839,9 +870,13 @@ public:
     void *asArray() const;
 
 #if wxUSE_STD_IOSTREAM
-    friend wxSTD ostream& operator<<(wxSTD ostream&, const wxULongLongWx&);
+    friend WXDLLIMPEXP_BASE
+    wxSTD ostream& operator<<(wxSTD ostream&, const wxULongLongWx&);
 #endif // wxUSE_STD_IOSTREAM
 
+    friend WXDLLIMPEXP_BASE
+    wxString& operator<<(wxString&, const wxULongLongWx&);
+
 private:
     // long is at least 32 bits, so represent our 64bit number as 2 longs
 
@@ -854,7 +889,7 @@ private:
         wxASSERT( (m_ll >> 32) == m_hi && (unsigned long)m_ll == m_lo );
     }
 
-    unsigned wxLongLong_t m_ll;
+    wxULongLong_t m_ll;
 #endif // wxLONGLONG_TEST_MODE
 };
 
@@ -886,10 +921,10 @@ inline bool operator!=(unsigned long l, const wxULongLong& ull) { return ull !=
 
 inline wxULongLong operator+(unsigned long l, const wxULongLong& ull) { return ull + l; }
 
-// FIXME: this should return wxLongLong
-inline wxULongLong operator-(unsigned long l, const wxULongLong& ull)
+inline wxLongLong operator-(unsigned long l, const wxULongLong& ull)
 {
-    return wxULongLong(l) - ull;
+    wxULongLong ret = wxULongLong(l) - ull;
+    return wxLongLong((long)ret.GetHi(),ret.GetLo());
 }
 
 #endif // _WX_LONGLONG_H
