@@ -1,5 +1,5 @@
-#if __WXMAC_CLASSIC__
+#ifdef __WXMAC_CLASSIC__
 #include "wx/mac/classic/pnghand.h"
 #else
 #include "wx/mac/carbon/pnghand.h"
-#endif
\ No newline at end of file
+#endif
