@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: VZ at 16/11/98: WX_DECLARE_LIST() and typesafe lists added
 // Created:     04/01/98
-// RCS-ID:      $Id: list.cpp,v 1.51 2004/09/17 17:57:44 ABX Exp $
+// RCS-ID:      $Id: list.cpp,v 1.54 2005/03/24 20:07:00 ABX Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 ////////////////////////////////////////////////////////////////////////////////
@@ -756,4 +756,15 @@ wxNode *wxStringList::Prepend(const wxChar *s)
 
 #endif // wxLIST_COMPATIBILITY
 
+#else // wxUSE_STL = 1
+
+    #include <wx/listimpl.cpp>
+    WX_DEFINE_LIST(wxObjectList);
+
+// with wxUSE_STL wxStringList contains wxString objects, not pointers
+void wxStringListBase::DeleteFunction( const wxString WXUNUSED(X) )
+{
+}
+
 #endif // !wxUSE_STL
+
