@@ -6,7 +6,7 @@
  *          Guillermo Rodriguez Garcia <guille@iies.es>
  * Purpose: GSocket main Unix and OS/2 file
  * Licence: The wxWindows licence
- * CVSID:   $Id: gsocket.cpp,v 1.24 2004/09/27 20:51:19 VZ Exp $
+ * CVSID:   $Id: gsocket.cpp,v 1.39 2005/03/28 19:46:01 SN Exp $
  * -------------------------------------------------------------------------
  */
 
@@ -101,6 +101,9 @@ int _System soclose(int);
 #ifdef sgi
 #  include <bstring.h>
 #endif
+#ifdef _AIX
+#  include <strings.h>
+#endif
 #include <signal.h>
 
 #ifndef SOCKLEN_T
@@ -117,14 +120,12 @@ int _System soclose(int);
 #  endif
 #endif
 
-#else 
-   /* undefine for OSX - its really an int */
-#  ifdef __DARWIN__
-#    undef SOCKLEN_T
-#    define SOCKLEN_T int
-#  endif
 #endif /* SOCKLEN_T */
 
+#ifndef SOCKOPTLEN_T
+#define SOCKOPTLEN_T SOCKLEN_T
+#endif
+
 /*
  * MSW defines this, Unices don't.
  */
@@ -155,6 +156,15 @@ int _System soclose(int);
   signal(SIGPIPE, old_handler);             \
 }
 
+/* If a SIGPIPE is issued by a socket call on a remotely closed socket,
+   the program will "crash" unless it explicitly handles the SIGPIPE.
+   By using MSG_NOSIGNAL, the SIGPIPE is suppressed. Later, we will
+   use SO_NOSIGPIPE (if available), the BSD equivalent. */
+#ifdef MSG_NOSIGNAL
+#  define GSOCKET_MSG_NOSIGNAL MSG_NOSIGNAL
+#else /* MSG_NOSIGNAL not available (FreeBSD including OS X) */
+#  define GSOCKET_MSG_NOSIGNAL 0
+#endif /* MSG_NOSIGNAL */
 
 #ifndef __GSOCKET_STANDALONE__
 #  include "wx/unix/gsockunx.h"
@@ -478,11 +488,13 @@ GSocketError GSocket::SetServer()
     m_error = GSOCK_IOERR;
     return GSOCK_IOERR;
   }
-#if defined(__EMX__) || defined(__VISAGECPP__)
-  ioctl(m_fd, FIONBIO, (char*)&arg, sizeof(arg));
-#else
-  ioctl(m_fd, FIONBIO, &arg);
+
+  /* FreeBSD variants can't use MSG_NOSIGNAL, and instead use a socket option */
+#ifdef SO_NOSIGPIPE
+  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(u_long));
 #endif
+
+  ioctl(m_fd, FIONBIO, &arg);
   gs_gui_functions->Enable_Events(this);
 
   /* allow a socket to re-bind if the socket is in the TIME_WAIT
@@ -531,9 +543,6 @@ GSocket *GSocket::WaitConnection()
 
   assert(this);
 
-  /* Reenable CONNECTION events */
-  Enable(GSOCK_CONNECTION);
-
   /* If the socket has already been created, we exit immediately */
   if (m_fd == INVALID_SOCKET || !m_server)
   {
@@ -560,6 +569,9 @@ GSocket *GSocket::WaitConnection()
 
   connection->m_fd = accept(m_fd, &from, (SOCKLEN_T *) &fromlen);
 
+  /* Reenable CONNECTION events */
+  Enable(GSOCK_CONNECTION);
+
   if (connection->m_fd == INVALID_SOCKET)
   {
     if (errno == EWOULDBLOCK)
@@ -586,7 +598,6 @@ GSocket *GSocket::WaitConnection()
   err = _GAddress_translate_from(connection->m_peer, &from, fromlen);
   if (err != GSOCK_NOERROR)
   {
-    GAddress_destroy(connection->m_peer);
     delete connection;
     m_error = err;
     return NULL;
@@ -672,16 +683,28 @@ GSocketError GSocket::Connect(GSocketStream stream)
     m_error = GSOCK_IOERR;
     return GSOCK_IOERR;
   }
+
+  /* FreeBSD variants can't use MSG_NOSIGNAL, and instead use a socket option */
+#ifdef SO_NOSIGPIPE
+  setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, (const char*)&arg, sizeof(u_long));
+#endif
+
 #if defined(__EMX__) || defined(__VISAGECPP__)
   ioctl(m_fd, FIONBIO, (char*)&arg, sizeof(arg));
 #else
   ioctl(m_fd, FIONBIO, &arg);
 #endif
-  gs_gui_functions->Enable_Events(this);
 
   /* Connect it to the peer address, with a timeout (see below) */
   ret = connect(m_fd, m_peer->m_addr, m_peer->m_len);
 
+  /* We only call Enable_Events if we know e aren't shutting down the socket */
+
+  if (m_non_blocking)
+  {
+    gs_gui_functions->Enable_Events(this);
+  }
+
   if (ret == -1)
   {
     err = errno;
@@ -702,9 +725,11 @@ GSocketError GSocket::Connect(GSocketStream stream)
       else
       {
         int error;
-        SOCKLEN_T len = sizeof(error);
+        SOCKOPTLEN_T len = sizeof(error);
 
-        getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (void*) &error, &len);
+        getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (char*) &error, &len);
+
+        gs_gui_functions->Enable_Events(this);
 
         if (!error)
           return GSOCK_NOERROR;
@@ -833,7 +858,7 @@ int GSocket::Read(char *buffer, int size)
 
   if (ret == -1)
   {
-    if (errno == EWOULDBLOCK)
+    if ((errno == EWOULDBLOCK) || (errno == EAGAIN))
       m_error = GSOCK_WOULDBLOCK;
     else
       m_error = GSOCK_IOERR;
@@ -877,7 +902,7 @@ int GSocket::Write(const char *buffer, int size)
 
   if (ret == -1)
   {
-    if (errno == EWOULDBLOCK)
+    if ((errno == EWOULDBLOCK) || (errno == EAGAIN))	  
     {
       m_error = GSOCK_WOULDBLOCK;
       GSocket_Debug(( "GSocket_Write error WOULDBLOCK\n" ));
@@ -959,7 +984,9 @@ GSocketEventFlags GSocket::Select(GSocketEventFlags flags)
     {
       char c;
 
-      if (recv(m_fd, &c, 1, MSG_PEEK) > 0)
+      int num = recv(m_fd, &c, 1, MSG_PEEK | GSOCKET_MSG_NOSIGNAL);
+
+      if (num > 0)
       {
         result |= GSOCK_INPUT_FLAG;
       }
@@ -970,7 +997,7 @@ GSocketEventFlags GSocket::Select(GSocketEventFlags flags)
           result |= GSOCK_CONNECTION_FLAG;
           m_detected |= GSOCK_CONNECTION_FLAG;
         }
-        else
+        else if ((errno != EWOULDBLOCK) && (errno != EAGAIN) && (errno != EINTR))
         {
           m_detected = GSOCK_LOST_FLAG;
           m_establishing = false;
@@ -987,11 +1014,11 @@ GSocketEventFlags GSocket::Select(GSocketEventFlags flags)
       if (m_establishing && !m_server)
       {
         int error;
-        SOCKLEN_T len = sizeof(error);
+        SOCKOPTLEN_T len = sizeof(error);
 
         m_establishing = false;
 
-        getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (void*)&error, &len);
+        getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (char*)&error, &len);
 
         if (error)
         {
@@ -1142,7 +1169,7 @@ void GSocket::UnsetCallback(GSocketEventFlags flags)
 GSocketError GSocket::GetSockOpt(int level, int optname,
                                 void *optval, int *optlen)
 {
-    if (getsockopt(m_fd, level, optname, optval, (SOCKLEN_T*)optlen) == 0)
+    if (getsockopt(m_fd, level, optname, (char*)optval, (SOCKOPTLEN_T*)optlen) == 0)
     {
         return GSOCK_NOERROR;
     }
@@ -1152,7 +1179,7 @@ GSocketError GSocket::GetSockOpt(int level, int optname,
 GSocketError GSocket::SetSockOpt(int level, int optname,
                                 const void *optval, int optlen)
 {
-    if (setsockopt(m_fd, level, optname, optval, optlen) == 0)
+    if (setsockopt(m_fd, level, optname, (const char*)optval, optlen) == 0)
     {
         return GSOCK_NOERROR;
     }
@@ -1271,7 +1298,12 @@ GSocketError GSocket::Output_Timeout()
 
 int GSocket::Recv_Stream(char *buffer, int size)
 {
-  return recv(m_fd, buffer, size, 0);
+  int ret;
+  do 
+  {
+    ret = recv(m_fd, buffer, size, GSOCKET_MSG_NOSIGNAL);
+  } while (ret == -1 && errno == EINTR); /* Loop until not interrupted */
+  return ret;
 }
 
 int GSocket::Recv_Dgram(char *buffer, int size)
@@ -1283,7 +1315,10 @@ int GSocket::Recv_Dgram(char *buffer, int size)
 
   fromlen = sizeof(from);
 
-  ret = recvfrom(m_fd, buffer, size, 0, &from, (SOCKLEN_T *) &fromlen);
+  do 
+  {
+    ret = recvfrom(m_fd, buffer, size, 0, &from, (SOCKLEN_T *) &fromlen);
+  } while (ret == -1 && errno == EINTR); /* Loop until not interrupted */
 
   if (ret == -1)
     return -1;
@@ -1315,11 +1350,14 @@ int GSocket::Send_Stream(const char *buffer, int size)
   int ret;
 
 #ifndef __VISAGECPP__
-  MASK_SIGNAL();
-  ret = send(m_fd, buffer, size, 0);
+   MASK_SIGNAL();
+#endif 
+  do 
+  {
+    ret = send(m_fd, (char *)buffer, size, GSOCKET_MSG_NOSIGNAL);
+  } while (ret == -1 && errno == EINTR); /* Loop until not interrupted */
+#ifndef __VISAGECPP__
   UNMASK_SIGNAL();
-#else
-  ret = send(m_fd, (char *)buffer, size, 0);
 #endif
 
   return ret;
@@ -1346,10 +1384,13 @@ int GSocket::Send_Dgram(const char *buffer, int size)
 
 #ifndef __VISAGECPP__
   MASK_SIGNAL();
-  ret = sendto(m_fd, buffer, size, 0, addr, len);
+#endif
+  do 
+  {
+    ret = sendto(m_fd, (char *)buffer, size, 0, addr, len);
+  } while (ret == -1 && errno == EINTR); /* Loop until not interrupted */
+#ifndef __VISAGECPP__
   UNMASK_SIGNAL();
-#else
-  ret = sendto(m_fd, (char *)buffer, size, 0, addr, len);
 #endif
 
   /* Frees memory allocated from _GAddress_translate_to */
@@ -1362,6 +1403,12 @@ void GSocket::Detected_Read()
 {
   char c;
 
+  /* Safeguard against straggling call to Detected_Read */
+  if (m_fd == INVALID_SOCKET)
+  {
+    return;
+  }
+
   /* If we have already detected a LOST event, then don't try
    * to do any further processing.
    */
@@ -1374,7 +1421,9 @@ void GSocket::Detected_Read()
     return;
   }
 
-  if (recv(m_fd, &c, 1, MSG_PEEK) > 0)
+  int num =  recv(m_fd, &c, 1, MSG_PEEK | GSOCKET_MSG_NOSIGNAL);
+
+  if (num > 0)
   {
     CALL_CALLBACK(this, GSOCK_INPUT);
   }
@@ -1386,8 +1435,16 @@ void GSocket::Detected_Read()
     }
     else
     {
-      CALL_CALLBACK(this, GSOCK_LOST);
-      Shutdown();
+      /* Do not throw a lost event in cases where the socket isn't really lost */
+      if ((errno == EWOULDBLOCK) || (errno == EAGAIN) || (errno == EINTR)) 
+      {
+        CALL_CALLBACK(this, GSOCK_INPUT);
+      }
+      else 
+      {
+        CALL_CALLBACK(this, GSOCK_LOST);
+        Shutdown();
+      } 
     }
   }
 }
@@ -1409,11 +1466,11 @@ void GSocket::Detected_Write()
   if (m_establishing && !m_server)
   {
     int error;
-    SOCKLEN_T len = sizeof(error);
+    SOCKOPTLEN_T len = sizeof(error);
 
     m_establishing = false;
 
-    getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (void*)&error, &len);
+    getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (char*)&error, &len);
 
     if (error)
     {
@@ -1648,7 +1705,7 @@ GSocketError GAddress_INET_SetHostName(GAddress *address, const char *hostname)
   if (inet_aton(hostname, addr) == 0)
   {
 #elif defined(HAVE_INET_ADDR)
-  if ( (addr->s_addr = inet_addr(hostname)) == -1 )
+  if ( (addr->s_addr = inet_addr(hostname)) == (unsigned)-1 )
   {
 #else
   /* Use gethostbyname by default */
@@ -1709,7 +1766,11 @@ GSocketError GAddress_INET_SetPortName(GAddress *address, const char *port,
     return GSOCK_INVPORT;
   }
 
+#if defined(__WXPM__) && defined(__EMX__)
+  se = getservbyname(port, (char*)protocol);
+#else
   se = getservbyname(port, protocol);
+#endif
   if (!se)
   {
     /* the cast to int suppresses compiler warnings about subscript having the
