@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     9-Dec-1999
-# RCS-ID:      $Id: buttons.py,v 1.9 2004/09/23 20:47:16 RD Exp $
+# RCS-ID:      $Id: buttons.py,v 1.10 2005/03/03 01:51:53 RD Exp $
 # Copyright:   (c) 1999 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -202,7 +202,9 @@ class GenButton(wx.PyControl):
         wx.PyControl.SetForegroundColour(self, colour)
         self.InitColours()
 
-
+    def SetDefault(self):
+        self.GetParent().SetDefaultItem(self)
+        
     def _GetLabelSize(self):
         """ used internally """
         w, h = self.GetTextExtent(self.GetLabel())
